/*-------------------------------------------------------------------
 *
 *  @file tracker.h
 *
 *  @author Collin R. Mulliner <collin@mulliner.org>
 *
 *  (c) Collin R. Mulliner
 *
 *  This is part of the Bluetooth Location Tracker (BLT) project.
 *
 *-------------------------------------------------------------------*/

/*
 * This file is part of blt_server
 *
 * blt_server is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * blt_server is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with blt_server; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
 
#ifndef __TRACKER_H__
#define __TRACKER_H__

#define BLT_TC_SLEEPING			0
#define BLT_TC_SLEEP				1
#define BLT_TC_DOINQ				2
#define BLT_TC_ASKNAME			3
#define BLT_TC_READ				4
#define BLT_TC_DISCONNECTED	5

#define BLUETOOTH_SLOT			1.28

#define BLT_TRACKER_NAME "BLT_#%d"

struct blt_tracker_config_t {

	unsigned char pageScan;					/**< 1 on / 0 off */
	unsigned char inquiryScan;				/**< 1 on / 0 off */
	
	unsigned short int inquiryDuration; /**< number of SLOTs */
	unsigned short int pageTimeout;		/**< number of SLOTs */
	
	unsigned short int maxNumDevices;	/**< max result devices */
	
	unsigned short int maxAskName;		/**< max ask name per inquiry */
	
	char *name;									/**< device name / NULL to use default */
};

struct blt_tracker_client_session_t {
	
	struct blt_server_session_t *server;	/**< pointer to global data */
	int myListIndex;								/**< index to trackers[] in server_session */
	pthread_t myThread;							/**< thread data */
	
	int mySocket;							/**< communication socket */
	struct sockaddr_in clientAddr;	/**< the clients address */
	socklen_t clientAl;
	
	unsigned char status;				/**< status of connection */
	unsigned int doInq;					/**< flag to switch to inquiry mode */
	unsigned int msgNo;					/**< message number */
	
	/* private store */
	struct blt_device_status_t *prvDevices;
	unsigned int numPrvDevices;
	unsigned int maxPrvDevices;
	
	/* public store */
	struct blt_device_status_t *pubDevices;
	unsigned int numPubDevices;
	pthread_rwlock_t pubDevicesLock;
	
	/* configuration */
	struct blt_tracker_config_t *cfg;
	
	/* signalling */
	pthread_cond_t actionSig;
	pthread_mutex_t actionLock;
	
	/* ask name stuff */
	unsigned int askNameCur;
	unsigned int askNameNum;
	
	SSL *ssl;
	
	struct blt_location_name_t loc;
};
#endif

int blt_tc_doInq(struct blt_tracker_client_session_t *tcs);
int blt_tc_parseInqResult(struct blt_tracker_client_session_t *tcs, char *buffer);
void blt_tc_prvToPub(struct blt_tracker_client_session_t *tcs);
int blt_tc_conf(struct blt_tracker_client_session_t *tcs, int inq);
void *blt_tracker_client(void *args);
void blt_tc_init(struct blt_tracker_client_session_t *tcs, struct blt_server_session_t *server);
int blt_tc_parseAskNameResult(struct blt_tracker_client_session_t *tcs, char *buffer);
int blt_tc_askName(struct blt_tracker_client_session_t *tcs);
